(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs/operators')) :
    typeof define === 'function' && define.amd ? define('angular-drag-element', ['exports', '@angular/core', 'rxjs/operators'], factory) :
    (global = global || self, factory(global['angular-drag-element'] = {}, global.ng.core, global.rxjs.operators));
}(this, function (exports, core, operators) { 'use strict';

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var DragElementDirective = /** @class */ (function () {
        function DragElementDirective(_elementRef, _renderer) {
            this._elementRef = _elementRef;
            this._renderer = _renderer;
            /**
             * \@description
             * Turn on/off drag option.
             *
             * \@usageNotes
             * 'x' axis for horizontal drag
             * 'y' axis for vertical drag
             * 'a' axis for all over document drag
             *  It's case sensitive (use lower case)
             *
             * \@default
             * true
             */
            this.dragElement = true;
            /**
             * \@description
             * Provide axis to drag.
             *
             * \@usageNotes
             * 'x' axis for horizontal drag
             * 'y' axis for vertical drag
             * 'a' axis for all over document drag
             *  It's case sensitive (use lower case)
             *
             * \@default
             * 'y'
             */
            this.dragAxis = "y";
            /**
             * \@description
             * Show drag icon or not.
             *
             * \@usageNotes
             * 'true' will show icon
             * 'false' will hide icon
             *
             * \@default
             * true
             */
            this.dragIcon = false;
            this._mouseDown = new core.EventEmitter();
            this._mouseMove = new core.EventEmitter();
            this._mouseup = new core.EventEmitter();
            this._mouseOut = new core.EventEmitter();
        }
        /**
         * @private
         * @param {?} event
         * @return {?}
         */
        DragElementDirective.prototype.onMouseDown = /**
         * @private
         * @param {?} event
         * @return {?}
         */
        function (event) {
            this._mouseDown.next(event);
        };
        /**
         * @private
         * @param {?} event
         * @return {?}
         */
        DragElementDirective.prototype.onMouseMove = /**
         * @private
         * @param {?} event
         * @return {?}
         */
        function (event) {
            this._mouseMove.next(event);
        };
        /**
         * @private
         * @param {?} event
         * @return {?}
         */
        DragElementDirective.prototype.onMouseUp = /**
         * @private
         * @param {?} event
         * @return {?}
         */
        function (event) {
            this._mouseup.next(event);
        };
        /**
         * @private
         * @param {?} event
         * @return {?}
         */
        DragElementDirective.prototype.onMouseOut = /**
         * @private
         * @param {?} event
         * @return {?}
         */
        function (event) {
            this._mouseOut.next(event);
        };
        /**
         * @return {?}
         */
        DragElementDirective.prototype.ngOnInit = /**
         * @return {?}
         */
        function () {
            var _this = this;
            if (this.dragElement) {
                this.setupEventSubscriber();
                this._mouseDragSubscribe = this._mouseDrag.subscribe((/**
                 * @param {?} position
                 * @return {?}
                 */
                function (position) {
                    // console.log(position);
                    _this.updateElementPosition(position);
                }));
            }
        };
        /**
         * @return {?}
         */
        DragElementDirective.prototype.ngAfterViewInit = /**
         * @return {?}
         */
        function () {
            if (this.dragElement) {
                this.setupData();
                this.setDragIcon();
            }
        };
        /**
         * @return {?}
         */
        DragElementDirective.prototype.ngOnDestroy = /**
         * @return {?}
         */
        function () {
            if (this._mouseDragSubscribe) {
                this._mouseDragSubscribe.unsubscribe();
            }
        };
        /**
         * @private
         * @return {?}
         */
        DragElementDirective.prototype.setupData = /**
         * @private
         * @return {?}
         */
        function () {
            if (this.dragAxis === "y") {
                this._dragPointerType = "n-resize";
            }
            else if (this.dragAxis === "x") {
                this._dragPointerType = "e-resize";
            }
            else if (this.dragAxis === "xy") {
                this._dragPointerType = "move";
            }
            else {
                this._dragPointerType = null;
            }
        };
        /**
         * @private
         * @return {?}
         */
        DragElementDirective.prototype.setupEventSubscriber = /**
         * @private
         * @return {?}
         */
        function () {
            var _this = this;
            this._mouseDrag = this._mouseDown.pipe(operators.map((/**
             * @param {?} event
             * @return {?}
             */
            function (event) {
                event.preventDefault();
                _this._initialPosition = {
                    left: _this._elementRef.nativeElement.style.left,
                    top: _this._elementRef.nativeElement.style.top
                };
                return {
                    left: event["clientX"] -
                        _this._elementRef.nativeElement.getBoundingClientRect().left,
                    top: event["clientY"] -
                        _this._elementRef.nativeElement.getBoundingClientRect().top
                };
            })), operators.flatMap((/**
             * @param {?} offset
             * @return {?}
             */
            function (offset) {
                return _this._mouseMove.pipe(operators.map((/**
                 * @param {?} event
                 * @return {?}
                 */
                function (event) {
                    return {
                        left: event["clientX"] - offset.left,
                        top: event["clientY"] - offset.top
                    };
                })), operators.takeUntil(_this._mouseup), operators.takeUntil(_this._mouseOut));
            })));
        };
        /**
         * @private
         * @return {?}
         */
        DragElementDirective.prototype.setDragIcon = /**
         * @private
         * @return {?}
         */
        function () {
            if (this.dragIcon) {
                /** @type {?} */
                var div = document.createElement("div");
                div.style.textAlign = "center";
                div.style.fontSize = " 1.5em";
                div.style.padding = "0.2em 0";
                div.style.color = "#1565c0";
                div.className = "drag-icon-wrapper";
                div.innerHTML = this.dragIconInnerText;
                this._renderer.appendChild(this._elementRef.nativeElement, div);
            }
            if (this._dragPointerType) {
                this._renderer.setStyle(this._elementRef.nativeElement, "cursor", this._dragPointerType);
            }
            if (this.dragElementClass) {
                this._renderer.addClass(this._elementRef.nativeElement, this.dragElementClass);
            }
        };
        /**
         * @private
         * @param {?} position
         * @return {?}
         */
        DragElementDirective.prototype.updateElementPosition = /**
         * @private
         * @param {?} position
         * @return {?}
         */
        function (position) {
            if (!this._dragPointerType) {
                return;
            }
            if (this.dragAxis === "x") {
                this._renderer.setStyle(this._elementRef.nativeElement, "left", position.left + "px");
                this._renderer.setStyle(this._elementRef.nativeElement, "top", this._initialPosition.top + "px");
            }
            else if (this.dragAxis === "y") {
                this._renderer.setStyle(this._elementRef.nativeElement, "left", this._initialPosition.left + "px");
                this._renderer.setStyle(this._elementRef.nativeElement, "top", position.top + "px");
            }
            else if (this.dragAxis === "xy") {
                this._renderer.setStyle(this._elementRef.nativeElement, "top", position.top + "px");
                this._renderer.setStyle(this._elementRef.nativeElement, "left", position.left + "px");
            }
        };
        DragElementDirective.decorators = [
            { type: core.Directive, args: [{
                        selector: "[ngx-drag-element]"
                    },] }
        ];
        /** @nocollapse */
        DragElementDirective.ctorParameters = function () { return [
            { type: core.ElementRef },
            { type: core.Renderer2 }
        ]; };
        DragElementDirective.propDecorators = {
            dragElement: [{ type: core.Input, args: ["ngx-drag-element",] }],
            dragElementClass: [{ type: core.Input }],
            dragAxis: [{ type: core.Input }],
            dragIcon: [{ type: core.Input }],
            dragIconInnerText: [{ type: core.Input }],
            onMouseDown: [{ type: core.HostListener, args: ["mousedown", ["$event"],] }],
            onMouseMove: [{ type: core.HostListener, args: ["mousemove", ["$event"],] }],
            onMouseUp: [{ type: core.HostListener, args: ["mouseup", ["$event"],] }],
            onMouseOut: [{ type: core.HostListener, args: ["mouseout", ["$event"],] }]
        };
        return DragElementDirective;
    }());
    if (false) {
        /**
         * \@description
         * Turn on/off drag option.
         *
         * \@usageNotes
         * 'x' axis for horizontal drag
         * 'y' axis for vertical drag
         * 'a' axis for all over document drag
         *  It's case sensitive (use lower case)
         *
         * \@default
         * true
         * @type {?}
         */
        DragElementDirective.prototype.dragElement;
        /**
         * \@description
         * Set style for drag element
         *
         * \@usageNotes
         * All css style can be applied
         *
         * \@default
         * undefined
         * @type {?}
         */
        DragElementDirective.prototype.dragElementClass;
        /**
         * \@description
         * Provide axis to drag.
         *
         * \@usageNotes
         * 'x' axis for horizontal drag
         * 'y' axis for vertical drag
         * 'a' axis for all over document drag
         *  It's case sensitive (use lower case)
         *
         * \@default
         * 'y'
         * @type {?}
         */
        DragElementDirective.prototype.dragAxis;
        /**
         * \@description
         * Show drag icon or not.
         *
         * \@usageNotes
         * 'true' will show icon
         * 'false' will hide icon
         *
         * \@default
         * true
         * @type {?}
         */
        DragElementDirective.prototype.dragIcon;
        /**
         * \@description
         * Specify drag icon element as innerText.
         *
         * \@usageNotes
         * Give icon element as innerText in "string" value
         * \@example '<i class="fa fa-arrows-v"></i>'
         *
         * \@default
         * undefined
         * @type {?}
         */
        DragElementDirective.prototype.dragIconInnerText;
        /**
         * @type {?}
         * @private
         */
        DragElementDirective.prototype._dragPointerType;
        /**
         * @type {?}
         * @private
         */
        DragElementDirective.prototype._initialPosition;
        /**
         * @type {?}
         * @private
         */
        DragElementDirective.prototype._mouseDown;
        /**
         * @type {?}
         * @private
         */
        DragElementDirective.prototype._mouseMove;
        /**
         * @type {?}
         * @private
         */
        DragElementDirective.prototype._mouseup;
        /**
         * @type {?}
         * @private
         */
        DragElementDirective.prototype._mouseOut;
        /**
         * @type {?}
         * @private
         */
        DragElementDirective.prototype._mouseDragSubscribe;
        /**
         * @type {?}
         * @private
         */
        DragElementDirective.prototype._mouseDrag;
        /**
         * @type {?}
         * @private
         */
        DragElementDirective.prototype._elementRef;
        /**
         * @type {?}
         * @private
         */
        DragElementDirective.prototype._renderer;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var DragElementModule = /** @class */ (function () {
        function DragElementModule() {
        }
        DragElementModule.decorators = [
            { type: core.NgModule, args: [{
                        declarations: [DragElementDirective],
                        imports: [],
                        exports: [DragElementDirective]
                    },] }
        ];
        return DragElementModule;
    }());

    exports.DragElementDirective = DragElementDirective;
    exports.DragElementModule = DragElementModule;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=angular-drag-element.umd.js.map
