import { ElementRef, Renderer2, AfterViewInit, OnInit, OnDestroy } from "@angular/core";
export declare class DragElementDirective implements OnInit, AfterViewInit, OnDestroy {
    private _elementRef;
    private _renderer;
    /**
     * @description
     * Turn on/off drag option.
     *
     * @usageNotes
     * 'x' axis for horizontal drag
     * 'y' axis for vertical drag
     * 'a' axis for all over document drag
     *  It's case sensitive (use lower case)
     *
     * @default
     * true
     */
    dragElement: boolean;
    /**
     * @description
     * Set style for drag element
     *
     * @usageNotes
     * All css style can be applied
     *
     * @default
     * undefined
     */
    dragElementClass: string;
    /**
     * @description
     * Provide axis to drag.
     *
     * @usageNotes
     * 'x' axis for horizontal drag
     * 'y' axis for vertical drag
     * 'a' axis for all over document drag
     *  It's case sensitive (use lower case)
     *
     * @default
     * 'y'
     */
    dragAxis: "y" | "x" | "xy";
    /**
     * @description
     * Show drag icon or not.
     *
     * @usageNotes
     * 'true' will show icon
     * 'false' will hide icon
     *
     * @default
     * true
     */
    dragIcon: boolean;
    /**
     * @description
     * Specify drag icon element as innerText.
     *
     * @usageNotes
     * Give icon element as innerText in "string" value
     * @example '<i class="fa fa-arrows-v"></i>'
     *
     * @default
     * undefined
     */
    dragIconInnerText: any;
    private _dragPointerType;
    private _initialPosition;
    private _mouseDown;
    private _mouseMove;
    private _mouseup;
    private _mouseOut;
    private _mouseDragSubscribe;
    private _mouseDrag;
    constructor(_elementRef: ElementRef, _renderer: Renderer2);
    private onMouseDown;
    private onMouseMove;
    private onMouseUp;
    private onMouseOut;
    ngOnInit(): void;
    ngAfterViewInit(): void;
    ngOnDestroy(): void;
    private setupData;
    private setupEventSubscriber;
    private setDragIcon;
    private updateElementPosition;
}
